@@ -2,17 +2,18 @@
 A module for working with asyncio.
 """
 
-
 # built-in
 from asyncio import AbstractEventLoop as _AbstractEventLoop
 from asyncio import CancelledError as _CancelledError
+from asyncio import Event as _Event
 from asyncio import all_tasks as _all_tasks
-
-# from asyncio import gather as _gather
+from asyncio import get_event_loop as _get_event_loop
 from contextlib import suppress as _suppress
 from typing import Any as _Any
 from typing import Awaitable as _Awaitable
+from typing import Coroutine as _Coroutine
 from typing import Optional as _Optional
+from typing import TypeVar as _TypeVar
 
 
 def shutdown_loop(eloop: _AbstractEventLoop) -> None:
@@ -29,6 +30,7 @@ def shutdown_loop(eloop: _AbstractEventLoop) -> None:
             # Give all tasks a chance to complete.
             with _suppress(KeyboardInterrupt, _CancelledError):
                 eloop.run_until_complete(task)
+                task.exception()
 
 
 def run_handle_interrupt(
@@ -47,3 +49,23 @@ def run_handle_interrupt(
         shutdown_loop(eloop)
 
     return result
+
+
+T = _TypeVar("T")
+
+
+def run_handle_stop(stop_sig: _Event, task: _Coroutine[None, None, T]) -> T:
+    """
+    Publish the stop signal on keyboard interrupt and wait for the task to
+    complete.
+    """
+
+    loop = _get_event_loop()
+    to_run = loop.create_task(task)
+
+    while True:
+        try:
+            return loop.run_until_complete(to_run)
+        except KeyboardInterrupt:  # pragma: nocover
+            print("Keyboard interrupt.")
+            stop_sig.set()
